# imports
from Tkinter import *
from os import mkdir
from threading import Timer

class ErrorMsg:
    def __init__(self, master):

        frame = Frame(master)
        frame.grid(padx=4, pady=4)

        # main window title & icon
        master.iconbitmap(bitmap="graal.ico")
        master.title(string="Gmap Generator")

        # error
        Label(frame, text="template.nw not found!", fg="darkred").grid(row=0)

        # button
        Button(frame, text="Okay", command=frame.quit).grid(row=1)

class GmapGen:
    def __init__(self, master):

        frame = Frame(master)
        frame.grid(padx=4, pady=4)

        # main window title & icon
        master.iconbitmap(bitmap="graal.ico")
        master.title(string="Gmap Generator")
        
        # input labels
        Label(frame, text="Prefix:").grid(sticky=E, row=0)
        Label(frame, text="Width:").grid(sticky=E, row=1)
        Label(frame, text="Height:").grid(sticky=E, row=2)
        Label(frame, text="Center X:").grid(sticky=E, row=3)
        Label(frame, text="Center Y:").grid(sticky=E, row=4)

        # inputs
        self.ePrefix  = Entry(frame, width=10)
        self.eWidth   = Entry(frame, width=3)
        self.eHeight  = Entry(frame, width=3)
        self.eCenterX = Entry(frame, width=3)
        self.eCenterY = Entry(frame, width=3)

        self.ePrefix.grid(sticky=W, row=0, column=1)
        self.eWidth.grid(sticky=W, row=1, column=1)
        self.eHeight.grid(sticky=W, row=2, column=1)
        self.eCenterX.grid(sticky=W,row=3, column=1)
        self.eCenterY.grid(sticky=W, row=4, column=1)

        self.eWidth.insert(END, 2)
        self.eHeight.insert(END, 2)
        self.eCenterX.insert(END, 1)
        self.eCenterY.insert(END, 1)

        # link checkbox
        self.bLinks = IntVar()
        self.cLinks = Checkbutton(frame, text="Add level links", var=self.bLinks)
        self.cLinks.grid(sticky=W, row=5, column=1)

        # error/message output label
        self.lOutput = Label(frame, text='Click "Generate" to create your Gmap!')
        self.lOutput.grid(row=6, columnspan=2, pady=4)

        # credits :D
        Label(frame, text="by *Crow").grid(sticky=S+W, row=7, column=0)

        # button
        self.bGenerate = Button(frame, text="Generate", command=self.GenerateMap)
        self.bGenerate.grid(sticky=E, row=7, column=1)

        # window size stuff
        frame.columnconfigure(0, minsize=76)
        frame.columnconfigure(1, minsize=140)

    # set back to default
    def ClearMsg(self):
        self.lOutput["fg"]   = "black"
        self.lOutput["text"] = 'Click "Generate" to create your Gmap!'

    # message output
    def Message(self, msg, col):
        self.lOutput["fg"]   = col
        self.lOutput["text"] = msg
        Timer(3, self.ClearMsg).start()

    # get level coords
    def GetCoords(self, x, y):
        if x >= self.mCenterX:
            xs = str(x - self.mCenterX).zfill(2)
        else:
            i = self.mCenterX - x - 1
            a = i % 26
            b = int(i / 26)
            xs = chr(97 + b) + chr(97 + a)

        if y >= self.mCenterY:
            ys = str(y - self.mCenterY).zfill(2)
        else:
            i = self.mCenterY - y - 1
            a = i % 26
            b = int(i / 26)
            ys = chr(97 + b) + chr(97 + a)

        return xs + "-" + ys

    # get level links
    def GetLinks(self, x, y):
        # we don't want links. return
        if self.addLinks == 0:
            return ""

        links = ""
        if x > 1:
            links += "LINK {0}_{1}.nw 0 0 1 64 61 playery\n".format(self.mPrefix, self.GetCoords(x - 1, y))
        if x < self.mWidth:
            links += "LINK {0}_{1}.nw 63 0 1 64 0 playery\n".format(self.mPrefix, self.GetCoords(x + 1, y))
        if y > 1:
            links += "LINK {0}_{1}.nw 0 0 64 1 playerx 61\n".format(self.mPrefix, self.GetCoords(x, y - 1))
        if y < self.mHeight:
            links += "LINK {0}_{1}.nw 0 63 64 1 playerx 0\n".format(self.mPrefix, self.GetCoords(x, y + 1))

        return links


    # button madness!
    def GenerateMap(self):
        # make sure we won't run into any issues
        self.addLinks = self.bLinks.get()
        self.mPrefix  = self.ePrefix.get()
        if len(self.mPrefix) < 1:
            self.Message("The map prefix is too short!", "darkred")
            return
        
        try:
            self.mWidth   = int(self.eWidth.get())
            self.mHeight  = int(self.eHeight.get())
            self.mCenterX = int(self.eCenterX.get())
            self.mCenterY = int(self.eCenterY.get())
        except ValueError:
            self.Message("Only numeric values please!", "darkred")
            return

        if (self.mWidth < 2 and self.mHeight < 2) or self.mWidth < 1 or self.mHeight < 1:
            self.Message("The specified map size is too small!", "darkred")
            return
        
        # no issues. continue! create dir first
        try:
            mkdir(self.mPrefix)
        except:
            pass

        # prepare the GMAP file
        gmap = "GRMAP001\nWIDTH {0}\nHEIGHT {1}\nLEVELNAMES\n".format(self.mWidth, self.mHeight)

        # create levels
        for y in range(self.mHeight):
            for x in range(self.mWidth):
                l = "{0}_{1}.nw".format(self.mPrefix, self.GetCoords(x + 1, y + 1))
                gmap += "\"{0}\"{1}".format(l, x < self.mWidth - 1 and "," or "")
                
                f = open("{0}/{1}".format(self.mPrefix, l), "w")
                f.write(lTemplate + self.GetLinks(x + 1, y + 1))
                f.close()

            gmap += "\n"

        # finish the GMAP file and save it
        gmap += "LEVELNAMESEND\n"
        f = open("{0}/{0}.gmap".format(self.mPrefix), "w")
        f.write(gmap)
        f.close()

        # success!
        self.Message("Gmap created successfully!", "darkgreen")


# main window & timer
root = Tk()

# open template.nw
try:
    f = open("template.nw", "r")
except IOError: # error window
    err = ErrorMsg(root)
    root.mainloop()
    exit()

# template
lTemplate = ""
l = f.readline()
while l != "":
    lTemplate += l
    l = f.readline()

f.close()

# main app
gen = GmapGen(root)
root.mainloop()
